# -*- coding: utf-8 -*-
from cpython.ref cimport PyObject
from libc.string cimport memcpy
from hummus.interface cimport *


cdef class ByteWriter:

    def __cinit__(self):
        self._handle = new PythonByteWriter(<PyObject*>self)

    def __dealloc__(self):
        if self._handle != NULL:
            del self._handle

    def write(self, chunk):
        # Default implementation does nothing.
        return 0


cdef public api size_t _hummus_bw_write(
        object self, const_Byte* data, int size):
    return self.write(data[:size])


cdef class ByteWriterWithPosition:

    def __cinit__(self):
        self._handle = new PythonByteWriterWithPosition(<PyObject*>self)

    def __dealloc__(self):
        if self._handle != NULL:
            del self._handle

    def write(self, chunk):
        # Default implementation does nothing.
        return 0

    def tell(self):
        # Default implementation does nothing.
        return 0


cdef public api size_t _hummus_bw_tell(object self):
    return self.tell()


cdef class ByteReader:

    def __cinit__(self):
        self._handle = new PythonByteReader(<PyObject*>self)

    def __dealloc__(self):
        if self._handle != NULL:
            del self._handle

    def read(self, size):
        # Default implementation does nothing.
        return ''

    def __bool__(self):
        # Default implementation does nothing.
        return False


cdef public api size_t _hummus_br_read(object self, Byte* data, int size):
    cdef bytes text = self.read(size)
    cdef Byte* content = text
    actual_size = len(text)
    memcpy(data, content, actual_size)
    return actual_size


cdef public api size_t _hummus_br_not_ended(object self):
    return not bool(self)


cdef class ByteReaderWithPosition:

    def __cinit__(self):
        self._handle = new PythonByteReaderWithPosition(<PyObject*>self)

    def __dealloc__(self):
        if self._handle != NULL:
            del self._handle

    def read(self, size):
        # Default implementation does nothing.
        print('read', size)
        return ''

    def __bool__(self):
        # Default implementation does nothing.
        return False

    def seek(self, offset, whence=0):
        # Default implementation does nothing.
        pass

    def tell(self):
        # Default implementation does nothing.
        return 0


cdef public api void _hummus_br_seek_from_begin(object self, size_t offset):
    self.seek(offset)


cdef public api void _hummus_br_seek_from_end(object self, size_t offset):
    cdef int reverse = offset
    self.seek(-reverse, 2)


cdef public api void _hummus_br_seek(object self, size_t offset):
    self.seek(offset, 1)


cdef public api size_t _hummus_br_tell(object self):
    return self.tell()



# -*- coding: utf-8 -*-
from hummus.utils cimport to_string
from hummus.writer cimport *
from hummus.page cimport *
from hummus.font import Font
from hummus.interface cimport *
from hummus.reader cimport PageInput


cdef class Context:

    def __enter__(self):
        return self

    def close(self):
        # Release the page context.
        self._writer.EndPageContentContext(self._handle)

        # Null the handles.
        self._writer = NULL
        self._handle = NULL

    def __exit__(self, *args):
        self.close()

    property box:

        def __get__(self):
            return self._page.box

        def __set__(self, value):
            self._page.box = value

    def add(self, element):
        element.add_to(self)

    def embed(self, element):
        element.embed_to(self)



# -*- coding: utf-8 -*-
from hummus.writer import Writer
from hummus.reader import Reader


def Document(source, mode='w'):
    if mode == 'w':
        return Writer(source)

    if mode == 'r':
        return Reader(source)



# -*- coding: utf-8 -*-
from .meta import version as __version__, description as __doc__  # NOQA
from .document import Document
from .page import Page
from .rectangle import Rectangle
from .font import Font
from .text import Text
from .image import Image


__all__ = [
    'Document',
    'Page',
    'Rectangle',
    'Font',
    'Text',
    'Image'
]



# -*- coding: utf-8 -*-
from hummus.interface cimport *
from hummus.reader cimport *
from hummus.writer cimport *
from hummus.stream import StreamByteReaderWithPosition
from hummus.rectangle cimport Rectangle, PDFRectangle
from hummus.page cimport *
from hummus.interface cimport *
from hummus.context cimport *


cdef class PageInput:
    """Represents a specific page of a parsed PDF document.

    This is lazily-evaluated for information. It can safely be used as
    a quick reference to a page in the parsed document.
    """

    def __cinit__(self, Reader parent, int index):
        self._index = index
        self._parent = parent

        if index >= len(parent):
            raise IndexError

    def __dealloc__(self):
        if self._handle != NULL:
            del self._handle

    property index:

        def __get__(self):
            return self._index

    property box:

        def __get__(self):
            cdef Rectangle result

            self._parse()

            result = Rectangle.__new__(Rectangle)
            result._handle = new PDFRectangle(self._handle.GetMediaBox())
            return result

    def _parse(self):
        cdef PDFObject* _obj

        if self._handle == NULL:
            _obj = self._parent._handle.ParsePage(self._index)
            self._handle = new PDFPageInput(&self._parent._handle, _obj)

    def embed_to(self, Context ctx):
        cdef PDFPageRange page_range
        cdef ResourcesDictionary* resources
        cdef PythonByteReaderWithPosition* stream_handle = NULL
        cdef ByteReaderWithPosition base_reader
        cdef int current

        # Construct a page range to extract from the PDF.
        # TODO: Allow specified range.
        page_range.mType = eRangeTypeSpecific
        page_range.mSpecificRanges = ((self.index, self.index),)

        # Set the stream to the beginning.
        current = self._parent._stream.tell()
        self._parent._stream.seek(0)

        # Pull out the low-level handle.
        base_reader = <ByteReaderWithPosition>self._parent._stream
        stream_handle = base_reader._handle

        # Extract information from the PDF.
        status, pages = ctx._writer.CreateFormXObjectsFromPDF(
            stream_handle, page_range, ePDFPageBoxMediaBox)

        # Prepare the context for the operation.
        ctx._handle.q()

        resources = &(ctx._page._handle.GetResourcesDictionary())
        ctx._handle.Do(resources.AddFormXObjectMapping(pages[0]))

        # Finish the operation.
        ctx._handle.Q()

        # Reset the stream.
        self._parent._stream.seek(current)


cdef class Reader:
    """Reads basic metadata from a PDF document.

    @param[in] source
        A file-like object or a filename of the PDF document to read.
    """

    def __cinit__(self, source):
        cdef PythonByteReaderWithPosition* stream_handle = NULL
        cdef ByteReaderWithPosition base_reader
        cdef int status

        if hasattr(source, 'read'):
            # Construct a streaming reader from the stream.
            self._stream = StreamByteReaderWithPosition(source)

        else:
            # Construct a streaming reader from the stream.
            self._stream = StreamByteReaderWithPosition(open(source, 'rb'))
            self._managed = True

        # Pull out the low-level handle.
        base_reader = <ByteReaderWithPosition>self._stream
        stream_handle = base_reader._handle

        # Extract page information from the PDF.
        status = self._handle.StartPDFParsing(stream_handle)
        if status < 0:
            raise ValueError('Not a valid PDF document', source)

    def __enter__(self):
        return self

    def __exit__(self, *args):
        if self._managed:
            self._stream.close()

    def __len__(self):
        cdef int count = self._handle.GetPagesCount()
        return count

    def __getitem__(self, index):
        return PageInput(self, index)



# -*- coding: utf-8 -*-
from libcpp.string cimport string


cdef inline string to_string(value):
    if not value:
        return None

    if isinstance(value, str):
        return <string>value.encode('utf8')

    return <string>value



# -*- coding: utf-8 -*-
from libcpp.string cimport string
from libcpp.pair cimport pair
from libcpp.list cimport list
from hummus.interface cimport *
from hummus.context cimport PageContentContext
from hummus.page cimport *


cdef extern from "PDFWriter.h" namespace "PDFPageRange":
    ctypedef int ERangeType

    ERangeType eRangeTypeAll
    ERangeType eRangeTypeSpecific


cdef extern from "PDFFormXObject.h":

    cdef cppclass PDFFormXObject:
        int GetObjectID()


cdef extern from "PDFWriter.h":

    ctypedef int EPDFVersion

    EPDFVersion ePDFVersion10
    EPDFVersion ePDFVersion11
    EPDFVersion ePDFVersion12
    EPDFVersion ePDFVersion13
    EPDFVersion ePDFVersion14
    EPDFVersion ePDFVersion15
    EPDFVersion ePDFVersion16
    EPDFVersion ePDFVersion17

    ctypedef int EPDFPageBox

    EPDFPageBox ePDFPageBoxMediaBox
    EPDFPageBox ePDFPageBoxCropBox
    EPDFPageBox ePDFPageBoxBleedBox
    EPDFPageBox ePDFPageBoxTrimBox
    EPDFPageBox ePDFPageBoxArtBox

    cdef cppclass PDFUsedFont:
        pass

    ctypedef pair[long, long] ULongAndULong
    ctypedef list[ULongAndULong] ULongAndULongList

    ctypedef long ObjectIDType
    ctypedef list[ObjectIDType] ObjectIDTypeList
    ctypedef pair[int, ObjectIDTypeList] EStatusCodeAndObjectIDTypeList

    cdef cppclass PDFPageRange:
        ERangeType mType
        ULongAndULongList mSpecificRanges

    cdef cppclass PDFWriter:

        int StartPDF(string filename, EPDFVersion)
        int EndPDF()

        int StartPDFForStream(PythonByteWriterWithPosition*, EPDFVersion)
        int EndPDFForStream()

        int Reset()

        PageContentContext* StartPageContentContext(PDFPage*)
        int EndPageContentContext(PageContentContext*)

        int WritePageAndRelease(PDFPage*)
        int WritePage(PDFPage*)

        PDFUsedFont* GetFontForFile(string filename, int index=0)

        PDFFormXObject* CreateFormXObjectFromJPGFile(const string&)

        PDFFormXObject* CreateFormXObjectFromJPGStream(
            PythonByteReaderWithPosition*)

        EStatusCodeAndObjectIDTypeList CreateFormXObjectsFromPDF(
            const string&, const PDFPageRange&, EPDFPageBox)

        EStatusCodeAndObjectIDTypeList CreateFormXObjectsFromPDF(
            PythonByteReaderWithPosition*, const PDFPageRange&, EPDFPageBox)



# -*- coding: utf-8 -*-
import six
from hummus import Font
from hummus.utils cimport *
from hummus.context cimport *


cdef class Text:
    cdef string _text
    cdef string _font
    cdef int _size
    cdef int _x
    cdef int _y

    def __cinit__(self, text, font='arial', size=10, x=0, y=0):
        # Ensure we have a valid font.
        if isinstance(font, six.string_types):
            font = Font(font)
            if not font:
                raise ValueError('Not a valid font family', font)

        # Store properties.
        self._text = to_string(text)
        self._font = to_string(font.file)
        self._size = size
        self._x = x
        self._y = y

    def add_to(self, Context ctx):
        cdef PDFUsedFont* font

        # Create a font object from the passed font.
        font = ctx._writer.GetFontForFile(self._font)
        if font is NULL:
            raise ValueError('Font not recognized by PDF', self._font)

        # Perform the write operation.
        ctx._handle.BT()
        ctx._handle.k(0, 0, 0, 1)
        ctx._handle.Tf(font, self._size)
        ctx._handle.Tm(1, 0, 0, 1, self._x, self._y)
        ctx._handle.Tj(self._text)
        ctx._handle.ET()



# -*- coding: utf-8 -*-


cdef extern from "PDFRectangle.h":

    cdef cppclass PDFRectangle:
        PDFRectangle(float x0, float y0, float x1, float y1)
        PDFRectangle(const PDFRectangle& rhs)

        float LowerLeftX
        float LowerLeftY
        float UpperRightX
        float UpperRightY


cdef class Rectangle:
    cdef PDFRectangle* _handle



# -*- coding: utf-8 -*-
from hummus.rectangle cimport *


cdef class Rectangle:

    def __init__(
            self,
            float left=0,
            float top=0,
            float right=0,
            float bottom=0):
        self._handle = new PDFRectangle(left, top, right, bottom)

    def __dealloc__(self):
        if self._handle != NULL:
            del self._handle

    property left:

        def __get__(self):
            return self._handle.LowerLeftX

        def __set__(self, value):
            self._handle.LowerLeftX = value

    property top:

        def __get__(self):
            return self._handle.LowerLeftY

        def __set__(self, value):
            self._handle.LowerLeftY = value

    property right:

        def __get__(self):
            return self._handle.UpperRightX

        def __set__(self, value):
            self._handle.UpperRightX = value

    property bottom:

        def __get__(self):
            return self._handle.UpperRightY

        def __set__(self, value):
            self._handle.UpperRightY = value



# -*- coding: utf-8 -*-
from hummus.interface import (ByteWriter, ByteWriterWithPosition,
                              ByteReader, ByteReaderWithPosition)


class StreamByteWriter(ByteWriter):

    def __init__(self, stream):
        #! The file-like object to send writes to.
        self._stream = stream

    def write(self, data):
        return self._stream.write(data)

    def close(self):
        self._stream.close()


class StreamByteWriterWithPosition(ByteWriterWithPosition):

    def __init__(self, stream):
        #! The file-like object to send writes to.
        self._stream = stream

    def write(self, data):
        return self._stream.write(data)

    def tell(self):
        return self._stream.tell()

    def close(self):
        self._stream.close()


class StreamByteReader(ByteReader):

    def __init__(self, stream):
        #! The file-like object to send reads to.
        self._stream = stream

        #! Have we hit the end of the stream.
        self._end = False

    def read(self, size):
        data = self._stream.read(size)
        if size != 0 and data == '':
            self._end = True
        return data

    def __bool__(self):
        return self._end

    def close(self):
        self._stream.close()


class StreamByteReaderWithPosition(ByteReaderWithPosition):

    def __init__(self, stream):
        #! The file-like object to send reads to.
        self._stream = stream

        #! Have we hit the end of the stream.
        self._end = False

    def read(self, size):
        data = self._stream.read(size)
        if size != 0 and len(data) == 0:
            self._end = True
        return data

    def __bool__(self):
        return self._end

    def seek(self, offset, whence=0):
        self._stream.seek(offset, whence)

    def tell(self):
        return self._stream.tell()

    def close(self):
        self._stream.close()



# -*- coding: utf-8 -*-
from hummus.interface cimport *
from hummus.rectangle cimport PDFRectangle


cdef extern from "PDFObject.h":

    cdef cppclass PDFObject:
        pass


cdef extern from "PDFPageInput.h":

    cdef cppclass PDFPageInput:
        PDFPageInput(PDFParser* inParser, PDFObject* inPageObject)
        PDFRectangle GetMediaBox()


cdef extern from "PDFParser.h":

    cdef cppclass PDFParser:
        int StartPDFParsing(PythonByteReaderWithPosition* source)
        int GetPagesCount()

        PDFObject* ParsePage(int index)


cdef class Reader:
    cdef PDFParser _handle
    cdef _stream
    cdef bint _managed


cdef class PageInput:
    cdef PDFPageInput* _handle
    cdef int _index
    cdef Reader _parent



# -*- coding: utf-8 -*-
from cpython.ref cimport PyObject


cdef extern from "IOBasicTypes.h":
    ctypedef unsigned char Byte "unsigned char"
    ctypedef unsigned char const_Byte "const unsigned char"

    ctypedef size_t LongBufferSizeType
    ctypedef long LongFilePositionType


cdef extern from "interface/PythonByteWriter.h":

    cdef cppclass PythonByteWriter:
        PythonByteWriter(PyObject* obj)
        LongBufferSizeType Write(const Byte* stream, LongBufferSizeType size)


cdef class ByteWriter:
    cdef PythonByteWriter* _handle


cdef extern from "interface/PythonByteWriterWithPosition.h":

    cdef cppclass PythonByteWriterWithPosition:
        PythonByteWriterWithPosition(PyObject* obj)
        LongBufferSizeType Write(const Byte* stream, LongBufferSizeType size)
        LongFilePositionType GetCurrentPosition()


cdef class ByteWriterWithPosition:
    cdef PythonByteWriterWithPosition* _handle


cdef extern from "interface/PythonByteReader.h":

    cdef cppclass PythonByteReader:
        PythonByteReader(PyObject* obj)
        LongBufferSizeType Read(Byte* stream, LongBufferSizeType size)
        bint NotEnded()


cdef class ByteReader:
    cdef PythonByteReader* _handle


cdef extern from "interface/PythonByteReaderWithPosition.h":

    cdef cppclass PythonByteReaderWithPosition:
        PythonByteReaderWithPosition(PyObject* obj)
        LongBufferSizeType Read(Byte* stream, LongBufferSizeType size)
        bint NotEnded()
        void SetPosition(LongFilePositionType offset)
        void SetPositionFromEnd(LongFilePositionType offset)
        void Skip(LongFilePositionType offset)
        LongFilePositionType GetCurrentPosition()


cdef class ByteReaderWithPosition:
   cdef PythonByteReaderWithPosition* _handle



# -*- coding: utf-8 -*-
from libcpp.string cimport string
from hummus.rectangle cimport PDFRectangle


cdef extern from "ResourcesDictionary.h":

    ctypedef long ObjectIDType

    cdef cppclass ResourcesDictionary:

        string AddFormXObjectMapping(ObjectIDType)


cdef extern from "PDFPage.h":

    cdef cppclass PDFPage:

        void SetMediaBox(const PDFRectangle& inMediaBox)
        PDFRectangle& GetMediaBox()

        ResourcesDictionary& GetResourcesDictionary()

cdef class Page:
    cdef PDFPage* _handle



# -*- coding: utf-8 -*-
import six
from contextlib import contextmanager
from libcpp.string cimport string
from hummus.writer cimport *
from hummus.utils cimport to_string
from hummus.interface cimport *
from hummus.stream import StreamByteWriterWithPosition
from hummus.page import Page
from hummus.context cimport PageContentContext, Context


cdef class Writer:
    cdef PDFWriter _handle
    cdef str _name
    cdef PythonByteWriterWithPosition* _stream

    def __cinit__(self, source=None):
        cdef PythonByteWriterWithPosition* stream_handle = NULL
        cdef ByteWriterWithPosition base_writer

        if hasattr(source, 'write'):
            # Construct a streaming writer.
            writer = StreamByteWriterWithPosition(source)

            # Pull out the low-level handle.
            base_writer = <ByteWriterWithPosition>writer
            stream_handle = base_writer._handle

            # Store the name and stream.
            self._stream = stream_handle
            self._name = ':memory:'

        else:
            # Store the filename.
            self._name = source

    def begin(self):
        """Begin operations on the PDF.
        """

        if self._stream:
            # Initiate the underlying PDF-Writer for streaming operation.
            self._handle.StartPDFForStream(self._stream, ePDFVersion17)

        else:
            # Initiate the underlying PDF-Writer for operations towards
            # a file.
            self._handle.StartPDF(to_string(self._name), ePDFVersion17)

    def __enter__(self):
        self.begin()
        return self

    def end(self):
        """End operations on the PDF.
        """

        if self._stream:
            # Terminate the streaming operations for the PDF-Writer.
            self._handle.EndPDFForStream()

        else:
            # Terminate the file operations for the PDF-W
            self._handle.EndPDF()

        # Reset the PDF-Writer for further operations.
        # self._handle.Reset()

    def __exit__(self, *args):
        self.end()

    property name:

        def __get__(self):
            """Get the name that this document is bound to.

            This returns the filename if the document is bound to a file or
            :memory: if it is bound to a stream.
            """
            return self._name

    def Context(self, Page page):
        cdef PageContentContext* handle
        cdef Context result

        # Create a content context for the passed page.
        handle = self._handle.StartPageContentContext(page._handle)

        # Wrap the handle.
        result = Context.__new__(Context)
        result._writer = &self._handle
        result._page = page
        result._handle = handle

        # Return our new context.
        return result

    def add(self, Page page):
        self._handle.WritePage(page._handle)

    @contextmanager
    def Page(self):
        cdef Page page
        cdef Context context

        # Instantiate a page and a context.
        page = Page()
        with self.Context(page) as context:
            # Yield the context.
            yield context

        # Add the page to the document.
        self.add(page)



# -*- coding: utf-8 -*-
import six
import io
from wand import image as wand
from hummus.utils cimport to_string
from hummus.context cimport *
from hummus.writer cimport *
from hummus.rectangle cimport *
from hummus.stream import StreamByteReaderWithPosition


cdef class Image:
    """Represents a PDF image that can be embedded in a document.

    Note that hummus can only take TIFF and JPEG images natively. However
    this transparently applies an image magick stream using wand to
    allow any image format to be embedded.
    """

    cdef _stream
    cdef Rectangle _box

    def __cinit__(self, source, index=0):
        # Ensure the source is a stream.
        temp_stream = io.BytesIO()

        # Construct a wand image stream over the source.
        key = 'filename' if isinstance(source, six.string_types) else 'file'
        with wand.Image(**{key: source}) as image:
            # Save the size of the image.
            self._box = Rectangle(bottom=image.height, right=image.width)

            # Set resolution units to undefined as PDF will embed a ghost
            # image if we do not. This is stupid by the way.
            image.units = 'undefined'
            image.resolution = 1
            # Convert the image to JPEG and save it into the byte stream.
            with wand.Image(image.sequence[index]) as o:
                o.format = 'jpeg'
                o.save(file=temp_stream)
                o.save(filename='something.jpg')

        # # Construct a streaming reader from the stream.
        temp_stream.seek(0)
        self._stream = StreamByteReaderWithPosition(temp_stream)

    def __enter__(self):
        return self

    def __exit__(self, *args):
        pass

    def embed_to(self, Context ctx):
        cdef PDFFormXObject* obj
        cdef ResourcesDictionary* resources
        cdef ByteReaderWithPosition base_reader
        cdef PythonByteReaderWithPosition* stream_handle = NULL

        # Create the PDF XObject from the stream.
        base_reader = <ByteReaderWithPosition>self._stream
        stream_handle = base_reader._handle
        obj = ctx._writer.CreateFormXObjectFromJPGStream(stream_handle)

        # Place the image on the page.
        ctx._handle.q()
        ctx._handle.cm(1, 0, 0, 1, 0, 0)
        resources = &(ctx._page._handle.GetResourcesDictionary())
        ctx._handle.Do(resources.AddFormXObjectMapping(obj.GetObjectID()))
        ctx._handle.Q()

    property box:

        def __get__(self):
            return self._box



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from setuptools import setup, find_packages, Extension
from pkgutil import get_importer
from collections import defaultdict
from functools import wraps
from fnmatch import fnmatch
from os.path import join
import os


def find(directory, patterns):
    result = []
    for node, _, filenames in os.walk(directory):
        for filename in filenames:
            for pattern in patterns:
                if fnmatch(filename, pattern):
                    result.append(join(node, filename))

    return result


def lazy(function):

    @wraps(function)
    def wrapped(*args, **kwargs):

        class LazyProxy(object):

            def __init__(self, function, args, kwargs):
                self._function = function
                self._args = args
                self._kwargs = kwargs
                self._result = None

            def __len__(self):
                return self.__len__()

            def __iter__(self):
                return self.__iter__()

            def __getattribute__(self, name):
                if name in ['_function', '_args', '_kwargs', '_result']:
                    return super(LazyProxy, self).__getattribute__(name)

                if self._result is None:
                    self._result = self._function(*self._args, **self._kwargs)

                return object.__getattribute__(self._result, name)

            def __setattr__(self, name, value):
                if name in ['_function', '_args', '_kwargs', '_result']:
                    super(LazyProxy, self).__setattr__(name, value)
                    return

                if self._result is None:
                    self._result = self._function(*self._args, **self._kwargs)

                setattr(self._result, name, value)

        return LazyProxy(function, args, kwargs)

    return wrapped


# Navigate, import, and retrieve the metadata of the project.
meta = get_importer('src/hummus').find_module('meta').load_module('meta')


def make_config():
    from pkgconfig import parse

    # Process the `pkg-config` utility and discover include and library
    # directories.
    config = defaultdict(set)
    for lib in ['zlib', 'libtiff-4', 'freetype2']:
        for key, value in parse(lib).items():
            config[key].update(value)

    # Add libjpeg (no .pc file).
    config['libraries'].add('jpeg')

    # List-ify config for setuptools.
    for key in config:
        config[key] = list(config[key])

    # Add hummus.
    config['include_dirs'].insert(0, 'lib/hummus/PDFWriter')
    config['include_dirs'].insert(0, 'lib/python')

    # Add local library.
    config['include_dirs'].insert(0, 'src')

    # Return built config.
    return config


@lazy
def make_extension(name, sources=None, cython=True):
    # Resolve extension location from name.
    location = join('src', *name.split('.'))
    location += '.pyx' if cython else '.cpp'

    # Create and return the extension.
    return Extension(
        name=name,
        sources=sources + [location] if sources else [location],
        language='c++',
        **make_config())


@lazy
def make_library(name, directory):
    patterns = ['*.cxx', '*.cpp']
    return [name, dict(sources=find(directory, patterns), **make_config())]


setup(
    name='hummus',
    version=meta.version,
    description=meta.description,
    author='Concordus Applications',
    author_email='support@concordusapps.com',
    url='https://github.com/concordusapps/python-hummus',
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'Intended Audience :: System Administrators',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 3.3',
    ],
    package_dir={'hummus': 'src/hummus'},
    packages=find_packages('src'),
    setup_requires=[
        'setuptools_cython',
        'pkgconfig'
    ],
    install_requires=[
        'six',
        'wand',
    ],
    extras_require={
        'test': ['pytest'],
    },
    libraries=[
        make_library('hummus', 'lib/hummus/PDFWriter'),
    ],
    ext_modules=[
        make_extension('hummus.reader'),
        make_extension('hummus.writer'),
        make_extension('hummus.rectangle'),
        make_extension('hummus.page'),
        make_extension('hummus.context'),
        make_extension('hummus.text'),
        make_extension('hummus.image'),
        make_extension(
            name='hummus.interface',
            sources=find('lib/python/interface', ['*.cxx'])),
    ]
)



# -*- coding: utf-8 -*-
from libcpp.string cimport string
from hummus.writer cimport PDFWriter, PDFUsedFont
from hummus.page cimport Page


cdef extern from "PageContentContext.h":

    cdef cppclass PageContentContext:

        void BT()
        void ET()

        void k(float cyan, float magneta, float yellow, float black)

        void Tm(float A, float B, float C, float D, float E, float F)

        void Tj(string)

        void Tf(PDFUsedFont*, float size)

        void q()
        void cm(float, float, float, float, float, float)
        void Q()

        void Do(string)

cdef class Context:
    cdef PDFWriter* _writer
    cdef Page _page
    cdef PageContentContext* _handle



# -*- coding: utf-8 -*-
from subprocess import Popen, PIPE


def find_font(family, weight="normal", slant="roman"):
    # Generate the query string for font-config.
    query = ':family=%s:weight=%s:slant=%s' % (family, weight, slant)

    # Ask font-config to see if we can match a font.
    process = Popen(['fc-match', '-s', query, 'file'], stdout=PIPE)
    out, _ = process.communicate()

    # Parse the output.
    out = out.decode('utf8')
    data = out.split()[0]
    if data:
        return data[6:]


class Font:

    def __init__(self, family, bold=False, italic=False):
        # Find the font.
        self.file = find_font(
            family,
            weight='bold' if bold else 'normal',
            slant='italic' if italic else 'roman')

        # Store properties of the font.
        self.family = family
        self.bold = bold
        self.italic = italic

    def __repr__(self):
        return '<Font(%r)>' % self.file

    def __bool__(self):
        return bool(self.file)



# -*- coding: utf-8 -*-
from cython.operator import dereference as deref
from hummus.page cimport *
from hummus.rectangle cimport Rectangle


cdef class Page:

    def __init__(self):
        self._handle = new PDFPage()

    def __dealloc__(self):
        if self._handle != NULL:
            del self._handle

    property box:

        def __get__(self):
            cdef PDFRectangle* handle
            cdef Rectangle result

            handle = <PDFRectangle*>&self._handle.GetMediaBox()
            result = Rectangle(handle.LowerLeftX, handle.LowerLeftY,
                               handle.UpperRightX, handle.UpperRightY)

            return result

        def __set__(self, Rectangle value):
            self._handle.SetMediaBox(deref(value._handle))
